import typer

from mediathequeroubaix import config, loans

app = typer.Typer(add_completion=False, no_args_is_help=True)
app.add_typer(config.app, name="config", no_args_is_help=True)
app.add_typer(loans.app, name="loans", no_args_is_help=True)


if __name__ == "__main__":
    app()



from returns.result import Success

from mediathequeroubaix.auth.get_user import get_user


def test_ok() -> None:
    html = """
        <a class="connect" href="/espace_personnel">\r
            <em class="icon icon-profile-white"></em>\r
            <div>\r
                <p>John DOE</p>\r
            </div>\r
    """

    assert get_user(html) == Success("John DOE")


def test_some_html_around() -> None:
    html = """
        <div>
            <p>some stuff</p>
        </div>
        ...
        <a class="connect" href="/espace_personnel">
            <em class="icon icon-profile-white"></em>
            <div>
                <p>John DOE</p>
            </div>
            ...
            <div>
                <p>some other stuff</p>
            </div>
    """

    assert get_user(html) == Success("John DOE")


def test_empty_html() -> None:
    html = ""

    assert str(get_user(html)) == "<Failure: User not found>"


def test_empty_user() -> None:
    html = r"""
        <a class="connect" href="/espace_personnel">
            <em class="icon icon-profile-white"></em>
            <div>
                <p></p>
            </div>
    """

    assert str(get_user(html)) == "<Failure: User not found>"



import re

from returns.result import Failure, Result, Success


def extract_token(s: str) -> Result[str, Exception]:
    match = re.search(r"<input.+id=\"my_issues\".+value=\"(.+)\"", s)
    if match:
        return Success(match.group(1))
    return Failure(ValueError("Token not found"))



from returns.pipeline import is_successful
from returns.result import Success

from mediathequeroubaix.get_loans.loan import Loan
from mediathequeroubaix.get_loans.parse_loans import parse_loans


def test_empty() -> None:
    assert is_successful(parse_loans("")) is False


def test_object() -> None:
    assert is_successful(parse_loans("{}")) is False


def test_empty_array() -> None:
    assert parse_loans("[]") == Success([])


def test_someloans() -> None:
    assert parse_loans(
        '[{"reasons_not_renewable": "too_many", "biblionumber": 298309, "issuedate": "2022-10-08T14:30:14", '
        '"barcode": "C2500002968", "branchcode": "MED", "itemcallnumber": "E BD/LEG", "date_due": "2022-11-19T23:59:00", "holdingbranch": "M\u00e9diath\u00e8que", "renewable": false, "borrowernumber": 48742, "title": "L\'\u00e9preuve d\'Had\u00e8s", "itemnumber": 360556}, '
        '{"barcode": "C2100005871", "biblionumber": 287420, "issuedate": "2020-01-02T03:04:05", "itemcallnumber": "R ABC", "date_due": "2020-01-02T03:04:06", "branchcode": "MED", "itemnumber": 341991, "holdingbranch": "M\u00e9diath\u00e8que", "renewable": true, "borrowernumber": 48742, "title": "Le secret d\'H\u00e9racl\u00e8s"}]'
    ) == Success(
        [
            Loan(
                title="L'épreuve d'Hadès",
                barcode="C2500002968",
                issuedate="2022-10-08T14:30:14",
                date_due="2022-11-19T23:59:00",
                itemcallnumber="E BD/LEG",
                renewable=False,
                reasons_not_renewable="too_many",
            ),
            Loan(
                title="Le secret d'Héraclès",
                barcode="C2100005871",
                issuedate="2020-01-02T03:04:05",
                date_due="2020-01-02T03:04:06",
                itemcallnumber="R ABC",
                renewable=True,
            ),
        ]
    )



from typing import Any

from requests_mock import mock
from returns.pipeline import is_successful
from returns.unsafe import unsafe_perform_io

from mediathequeroubaix.auth.authenticate import authenticate
from mediathequeroubaix.config import User


def match_user_pass(request: Any) -> bool:
    return "name=myuser&pass=mypass&form_id=user_login_block" in request.text


def test_ok(requests_mock: mock) -> None:
    requests_mock.post(
        "http://www.mediathequederoubaix.fr/user",
        additional_matcher=match_user_pass,
        text="""
            <a class="connect" href="/espace_personnel">
                <em class="icon icon-profile-white"></em>
                <div>
                    <p>John DOE</p>
                </div>
        """,
    )

    result = authenticate(User(login="myuser", password="mypass"))

    assert is_successful(result)
    actual = unsafe_perform_io(result.unwrap())
    assert actual.username == "John DOE"


def test_login_failure(requests_mock: mock) -> None:
    requests_mock.post("http://www.mediathequederoubaix.fr/user", text="some html")

    result = authenticate(User(login="myuser", password="mypass"))

    assert is_successful(result) is False



from returns.result import Success

from mediathequeroubaix.get_loans.base64decode import base64decode


def test_ok() -> None:
    assert base64decode("YWJj") == Success("abc")


def test_empty() -> None:
    assert base64decode("") == Success("")


def test_invalid() -> None:
    assert str(base64decode("[]{}")) == "<Failure: Non-base64 digit found>"



from datetime import date, datetime

from pydantic import validator
from pydantic.main import BaseModel

from mediathequeroubaix.auth.authenticated_session import Username


class Loan(BaseModel):
    # Vampi
    title: str
    # C2500011705
    barcode: str
    # 2022-10-08T14:30:35
    issuedate: str
    # 2022-11-19T23:59:00
    date_due: date
    # E BD/FON
    itemcallnumber: str
    # Boolean
    renewable: bool
    # Values: too_many, booked
    reasons_not_renewable: str | None

    @validator("date_due", pre=True)
    def parse_date_due(cls, value: str) -> date:
        return datetime.strptime(value, "%Y-%m-%dT%H:%M:%S").date()


class Loans(BaseModel):
    username: Username
    items: list[Loan]



import requests
from requests import Response, Session
from returns.io import IOResultE, impure_safe
from returns.pipeline import flow
from returns.pointfree import bind_result
from returns.result import safe

from mediathequeroubaix.auth.authenticated_session import AuthenticatedSession
from mediathequeroubaix.auth.get_user import get_user
from mediathequeroubaix.config import User


def authenticate(user: User) -> IOResultE[AuthenticatedSession]:
    session = requests.Session()
    return flow(
        (session, user.login, user.password),
        _connect,
        bind_result(_get_html),
        bind_result(get_user),
        bind_result(safe(lambda user: AuthenticatedSession(session, user))),
    )


@impure_safe
def _connect(data: tuple[Session, str, str]) -> Response:
    session, username, password = data
    response = session.post(
        "http://www.mediathequederoubaix.fr/user",
        data={
            "name": username,
            "pass": password,
            "form_id": "user_login_block",
        },
    )
    response.raise_for_status()
    return response


@safe
def _get_html(response: Response) -> str:
    return response.text



from pathlib import Path

import typer
from pydantic import BaseModel, BaseSettings
from returns.io import IOFailure, IOResultE, IOSuccess
from returns.pipeline import flow
from returns.pointfree import bind_ioresult
from returns.result import Success
from rich import print
from rich.panel import Panel
from rich.pretty import Pretty

APP_NAME = "mediathequeroubaix"
app = typer.Typer()


@app.command()
def show() -> None:
    match get_config():
        case IOSuccess(Success(config)):
            _show(_get_path(), config)
        case IOFailure(failure):
            print(failure)
            raise typer.Exit(code=1)


@app.command()
def create() -> None:
    config_path = _get_path()
    if config_path.is_file():
        print()
        print(
            f"[bold][yellow] Config file already exists:[/yellow] {_get_path()}[/bold]"
        )
        print()
        raise typer.Exit(code=1)
    else:
        config = _create_sample_config(config_path)
        _show(config_path, config)


class User(BaseModel):
    login: str
    password: str


class Config(BaseSettings):
    users: list[User] = []

    class Config:
        env_prefix = "MR_"  # defaults to no prefix


def get_config() -> IOResultE[Config]:
    return flow(
        _get_path(),
        _is_readable,
        bind_ioresult(_read),
    )


def _is_readable(path: Path) -> IOResultE[Path]:
    if path.is_file():
        return IOSuccess(path)
    return IOFailure(ValueError(f"File {path} is missing or not readable"))


def _get_path() -> Path:
    app_path = Path(typer.get_app_dir(APP_NAME))
    return app_path / "config.json"


def _read(path: Path) -> IOResultE[Config]:
    try:
        return IOSuccess(Config.parse_file(path))
    except Exception:
        return IOFailure(ValueError(f"Invalid configuration: {path}"))


def _show(config_path: Path, config: Config) -> None:
    print()
    print(f"[bold][yellow] Config:[/yellow] {config_path}[/bold]")
    print()
    panel = Panel(Pretty(config, expand_all=True))
    print(panel)
    print()


def _create_sample_config(config_path: Path) -> Config:
    config_path.parent.mkdir(parents=True, exist_ok=True)
    sample_config = Config(users=[User(login="X001002003", password="password00")])
    config_path.write_text(sample_config.json())
    return sample_config



from returns.io import IOResultE, impure_safe
from returns.pipeline import flow
from returns.pointfree import bind_result
from returns.result import safe

from mediathequeroubaix.auth.authenticated_session import AuthenticatedSession
from mediathequeroubaix.get_loans.base64decode import base64decode
from mediathequeroubaix.get_loans.extract_token import extract_token
from mediathequeroubaix.get_loans.loan import Loans
from mediathequeroubaix.get_loans.parse_loans import parse_loans


def get_loans(session: AuthenticatedSession) -> IOResultE[Loans]:
    return flow(
        session,
        _get_personal_space,
        bind_result(extract_token),
        bind_result(base64decode),
        bind_result(parse_loans),
        bind_result(safe(lambda loans: Loans(username=session.username, items=loans))),
    )


@impure_safe
def _get_personal_space(session: AuthenticatedSession) -> str:
    response = session.session.get(
        "http://www.mediathequederoubaix.fr/espace_personnel"
    )
    response.raise_for_status()
    return response.text



import datetime

from mediathequeroubaix.get_loans.loan import Loan


def test_convert_due_date_to_date() -> None:
    loan = Loan(
        title="L'épreuve d'Hadès",
        barcode="C2500002968",
        issuedate="2010-01-02T14:30:14",
        date_due="2023-04-05T23:59:00",
        itemcallnumber="E BD/LEG",
        renewable=False,
    )

    assert loan.date_due == datetime.date.fromisoformat("2023-04-05")



import typer
from returns.io import IOFailure, IOSuccess, impure_safe
from returns.pipeline import flow
from returns.pointfree import bind_ioresult, bind_result
from returns.result import Failure, ResultE, Success
from rich import box, print
from rich.table import Table

from mediathequeroubaix.auth.authenticate import authenticate
from mediathequeroubaix.config import Config, User, get_config
from mediathequeroubaix.get_loans.get_loans import get_loans
from mediathequeroubaix.get_loans.loan import Loans

app = typer.Typer()


@app.command(name="list")
def list_loans() -> None:
    loans = flow(
        get_config(),
        bind_result(_get_first_user),
        bind_ioresult(_print_user),
        bind_ioresult(authenticate),
        bind_ioresult(get_loans),
    )

    match loans:
        case IOSuccess(Success(loans)):
            _pretty_print(loans)
        case IOFailure(failure):
            print("❌ FAILURE!", failure)


def _get_first_user(config: Config) -> ResultE[User]:
    if config.users:
        # We only support having one user
        first_user = config.users[0]
        return Success(first_user)
    return Failure(ValueError("No user defined in configuration"))


@impure_safe
def _print_user(user: User) -> User:
    print(f"Getting loans of {user.login}")
    return user


def _pretty_print(loans: Loans) -> None:
    table = Table(
        title=f"{loans.username}: {len(loans.items)} loans",
        title_style="bold magenta",
        box=box.HEAVY_EDGE,
        expand=True,
    )
    table.add_column("#", justify="center")
    table.add_column("Title", style="blue", ratio=2)
    table.add_column("Due date", justify="center")
    table.add_column("Renewable", justify="center")

    for index, loan in enumerate(loans.items):
        renewable = "✅" if loan.renewable else "❌"
        table.add_row(
            f"{index+1:>2}/{len(loans.items)}",
            loan.title,
            f"{loan.date_due:%d/%m/%Y}",
            renewable,
        )

    print(table)
